@@ -2,9 +2,11 @@ from __future__ import annotations
 from functools import wraps as functools_wraps
 import inspect
 from enum import Enum
+from pathlib import Path
 from dataclasses import is_dataclass
-from typing import Any, Callable
+from typing import Any
 from typing_extensions import Annotated, _AnnotatedAlias
+from macrokit import Expr, register_type, symbol, Head
 
 from .gui.class_gui import (
     ClassGuiBase, 
@@ -21,12 +23,23 @@ from .gui.class_gui import (
     ToolBoxClassGui,
     ListClassGui,
     )
-from .gui._base import PopUpMode, ErrorMode, defaults, MagicTemplate
+from .gui._base import PopUpMode, ErrorMode, defaults, MagicTemplate, check_override
 from .gui import ContextMenuGui, MenuGui, MenuGuiBase
-from .macro import Expr
 from .utils import iter_members
 from ._app import get_app
 
+register_type(Enum, lambda e: repr(str(e.name)))
+register_type(Path, lambda e: f"r'{e}'")
+
+def find_myname(gui: MagicTemplate):
+    parent = gui.__magicclass_parent__
+    if parent is None:
+        return gui._my_symbol
+    else:
+        return Expr(Head.getattr, [find_myname(parent), gui._my_symbol])
+
+register_type(MagicTemplate, find_myname)
+
 _BASE_CLASS_SUFFIX = "_Base"
 
 class WidgetType(Enum):
@@ -90,7 +103,7 @@ def magicclass(class_: type|None = None,
                error_mode: str | ErrorMode = None,
                widget_type: str | WidgetType = WidgetType.none,
                parent = None
-               ) -> ClassGui | Callable[[type], ClassGui]:
+               ):
     """
     Decorator that can convert a Python class into a widget.
     
@@ -136,6 +149,9 @@ def magicclass(class_: type|None = None,
         close_on_run = defaults["close_on_run"]
     if error_mode is None:
         error_mode = defaults["error_mode"]
+    
+    if isinstance(widget_type, str):
+        widget_type = widget_type.lower()
         
     widget_type = WidgetType(widget_type)
     
@@ -149,6 +165,7 @@ def magicclass(class_: type|None = None,
         
         if not issubclass(cls, MagicTemplate):
             _check_collision(cls, class_gui)
+            check_override(cls)
             
         # get class attributes first
         doc = cls.__doc__
@@ -172,7 +189,7 @@ def magicclass(class_: type|None = None,
         @functools_wraps(oldclass.__init__)
         def __init__(self: MagicTemplate, *args, **kwargs):
             app = get_app() # Without "app = " Jupyter freezes after closing the window!
-            macro_init = Expr.parse_init(self, cls, args, kwargs)
+            
             class_gui.__init__(self, 
                                layout=layout,
                                parent=parent,
@@ -189,7 +206,9 @@ def magicclass(class_: type|None = None,
                 self.__post_init__()
             
             # Record class instance construction
+            macro_init = Expr.parse_init(self, cls, args, kwargs)
             self._recorded_macro.append(macro_init)
+            
             if widget_type in (WidgetType.collapsible, WidgetType.button):
                 self.btn_text = self.name
 
@@ -210,7 +229,7 @@ def magicmenu(class_: type = None,
               error_mode: str | ErrorMode = None,
               labels: bool = True, 
               parent = None
-              ) -> MenuGui | Callable[[type], MenuGui]:
+              ):
     """
     Decorator that can convert a Python class into a menu bar.
     """    
@@ -223,7 +242,7 @@ def magiccontext(class_: type = None,
                  error_mode: str | ErrorMode = None,
                  labels: bool = True, 
                  parent=None
-                 ) -> ContextMenuGui | Callable[[type], ContextMenuGui]:
+                 ):
     """
     Decorator that can convert a Python class into a context menu.
     """    
@@ -319,6 +338,7 @@ def _call_magicmenu(class_: type = None,
         
         if not issubclass(cls, MagicTemplate):
             _check_collision(cls, menugui_class)
+            check_override(cls)
         
         # get class attributes first
         doc = cls.__doc__
@@ -337,7 +357,7 @@ def _call_magicmenu(class_: type = None,
         @functools_wraps(oldclass.__init__)
         def __init__(self: MagicTemplate, *args, **kwargs):
             app = get_app() # Without "app = " Jupyter freezes after closing the window!
-            macro_init = Expr.parse_init(self, cls, args, kwargs)
+
             menugui_class.__init__(self, 
                                    parent=parent,
                                    close_on_run=close_on_run,
@@ -352,6 +372,7 @@ def _call_magicmenu(class_: type = None,
                 self.__post_init__()
             
             # Record class instance construction
+            macro_init = Expr.parse_init(self, cls, args, kwargs)
             self._recorded_macro.append(macro_init)
 
         newclass.__init__ = __init__
