@@ -10,11 +10,13 @@ from collections.abc import MutableSequence
 from magicgui.events import Signal
 from magicgui.signature import MagicParameter
 from magicgui.widgets import FunctionGui, FileEdit, EmptyWidget, Widget, Container
+from magicgui.widgets._bases import ValueWidget
+from macrokit import Macro, Expr, Head, Symbol, symbol
 
 from .keybinding import as_shortcut
 from .mgui_ext import AbstractAction, FunctionGuiPlus, PushButtonPlus
+from .utils import get_parameters
 
-from ..macro import Macro, Expr, Head, Symbol, symbol
 from ..utils import get_signature, iter_members, extract_tooltip, screen_center
 from ..widgets import Separator, MacroEdit
 from ..fields import MagicField
@@ -22,10 +24,8 @@ from ..signature import MagicMethodSignature, get_additional_option
 from ..wrappers import upgrade_signature
 
 if TYPE_CHECKING:
-    try:
-        import napari
-    except ImportError:
-        pass
+    import napari
+    from qtpy.QtWidgets import QDockWidget
 
 class PopUpMode(Enum):
     popup = "popup"
@@ -45,14 +45,32 @@ defaults = {"popup_mode": PopUpMode.popup,
             "close_on_run": True,
             }
 
+_RESERVED = {"__magicclass_parent__", "__magicclass_children__", "_close_on_run", 
+             "_error_mode", "_popup_mode", "_recorded_macro", "_my_symbol", "_macro_instance", 
+             "annotation", "enabled", "gui_only", "height", "label_changed", "label", "layout",
+             "labels", "margins", "max_height", "max_width", "min_height", "min_width", "name",
+             "options", "param_kind", "parent_changed", "tooltip", "visible", "widget_type", 
+             "width", "parent_viewer", "parent_dock_widget", "objectName", "create_macro", "wraps",
+             "_unwrap_method", "_search_parent_magicclass", "_iter_child_magicclasses",
+             }
+
+UI = Symbol("ui")
+
+def check_override(cls: type):
+    subclass_members = set(cls.__dict__.keys())
+    collision = subclass_members & _RESERVED
+    if collision:
+        raise AttributeError(f"Cannot override magic class reserved attributes: {collision}")
+
 class MagicTemplate:    
     __magicclass_parent__: None | MagicTemplate
     __magicclass_children__: list[MagicTemplate]
     _close_on_run: bool
     _component_class: type[AbstractAction | Widget]
     _error_mode: ErrorMode
+    _macro_instance: Macro
+    _my_symbol: Symbol
     _popup_mode: PopUpMode
-    _recorded_macro: Macro[Expr]
     annotation: Any
     changed: Signal
     enabled: bool
@@ -77,6 +95,8 @@ class MagicTemplate:
     widget_type: str
     width: int
     
+    __init_subclass__ = check_override
+    
     def show(self) -> None:
         raise NotImplementedError()
     
@@ -107,38 +127,17 @@ class MagicTemplate:
     def insert(self, key: int, widget: Widget) -> None:
         raise NotImplementedError()
     
-    def _collect_macro(self, parent_symbol: Symbol = None, self_symbol: Symbol = None) -> Macro:
-        if self_symbol is None:
-            self_symbol = symbol(self)
-        
-        if parent_symbol is not None:
-            self_symbol = Expr(Head.getattr, [parent_symbol, self_symbol])
-        
-        sym = symbol(self)
-        macro = Macro()
-        for expr in self._recorded_macro:
-            if parent_symbol is not None and expr.head == Head.init:
-                continue
-            macro.append(expr.format({sym: self_symbol}))
-        
-        for name, clsattr in iter_members(self.__class__):
-            # Collect all the macro from child magic-classes recursively
-            if not isinstance(clsattr, (MagicTemplate, type, MagicField)):
-                continue
-            
-            attr = getattr(self, name)
-            
-            if not isinstance(attr, MagicTemplate):
-                continue
-            
-            macro += attr._collect_macro(self_symbol, Symbol(name))
-                
-        return macro
+    @property
+    def _recorded_macro(self):
+        if self.__magicclass_parent__ is None:
+            return self._macro_instance
+        else:
+            return self.__magicclass_parent__._recorded_macro
     
     @property
     def parent_viewer(self) -> "napari.Viewer" | None:
         """
-        Return napari.Viewer if self is a dock widget of a viewer.
+        Return napari.Viewer if magic class is a dock widget of a viewer.
         """
         parent_self = self._search_parent_magicclass()
         try:
@@ -147,6 +146,19 @@ class MagicTemplate:
             viewer = None
         return viewer
     
+    @property
+    def parent_dock_widget(self) -> "QDockWidget" | None:
+        """
+        Return dock widget object if magic class is a dock widget of a main window widget,
+        such as a napari Viewer.
+        """
+        parent_self = self._search_parent_magicclass()
+        try:
+            dock = parent_self.native.parent()
+        except AttributeError:
+            dock = None
+        return dock
+    
     def objectName(self) -> str:
         """
         This function makes the object name discoverable by napari's 
@@ -154,7 +166,7 @@ class MagicTemplate:
         """        
         return self.native.objectName()
     
-    def create_macro(self, show: bool = False, myname: str = "ui") -> str:
+    def create_macro(self, show: bool = False) -> str:
         """
         Create executable Python scripts from the recorded macro object.
 
@@ -162,31 +174,9 @@ class MagicTemplate:
         ----------
         show : bool, default is False
             Launch a TextEdit window that shows recorded macro.
-        myname : str, default is "ui"
-            Symbol of the instance.
         """
-        # Recursively build macro from nested magic-classes
-        macro = self._collect_macro(self_symbol=Symbol(myname))
-        # Sort by the recorded order
-        sorted_macro = Macro(sorted(macro, key=lambda x: x.number))
         
-        script = str(sorted_macro)
-        # type annotation for the hard-to-record types
-        annot = []
-        idt_list = []
-        for expr in self._recorded_macro:
-            if expr.head == Head.init:
-                idt_list.append(expr.args[0].args[0])
-            for sym in expr.iter_args():
-                if sym.valid or sym in idt_list:
-                    continue
-                idt_list.append(sym)
-                annot.append(f"# {sym}: {sym.type}")
-        
-        if annot:
-            out = "\n".join(annot) + "\n" + script
-        else:
-            out = script
+        out = str(self._recorded_macro)
                     
         if show:
             win = MacroEdit(name="macro")
@@ -292,9 +282,10 @@ class MagicTemplate:
                         widget = child_instance._create_widget_from_method(lambda x: None)
                         child_instance.append(widget)
                     else:
-                        child_widget: PushButtonPlus = child_instance[index]
+                        child_widget: PushButtonPlus | AbstractAction = child_instance[index]
                         child_widget.changed.disconnect()
                         child_widget.changed.connect(widget.changed)
+                        child_widget.tooltip = widget.tooltip
                 break
         else:
             raise RuntimeError(f"{child_clsname} not found in class {self.__class__.__name__}")
@@ -330,8 +321,19 @@ class MagicTemplate:
             wrapper = _identity_wrapper
         else:
             raise ValueError(self._error_mode)
-            
-        func = wrapper(obj, parent=self)
+        
+        # Wrap function to block macro recording.
+        _inner_func = wrapper(obj, parent=self)
+        if _need_record(obj):
+            @functools_wraps(obj)
+            def func(*args, **kwargs):
+                with self._recorded_macro.blocked():
+                    out = _inner_func(*args, **kwargs)
+                return out
+        else:
+            @functools_wraps(obj)
+            def func(*args, **kwargs):
+                return _inner_func(*args, **kwargs)
         
         # Signature must be updated like this. Otherwise, already wrapped member function
         # will have signature with "self".
@@ -500,8 +502,13 @@ class MagicTemplate:
         # keybinding
         keybinding = get_additional_option(func, "keybinding", None)
         if keybinding is not None:
-            shortcut = as_shortcut(keybinding)
-            widget.set_shortcut(shortcut)
+            if obj.__name__.startswith("_"):
+                from qtpy.QtWidgets import QShortcut
+                shortcut = QShortcut(as_shortcut(keybinding), self.native)
+                shortcut.activated.connect(widget.changed)
+            else:
+                shortcut = as_shortcut(keybinding)
+                widget.set_shortcut(shortcut)
             
         return widget
     
@@ -518,14 +525,15 @@ class MagicTemplate:
     
 class BaseGui(MagicTemplate):
     def __init__(self, close_on_run, popup_mode, error_mode):
-        self._recorded_macro: Macro[Expr] = Macro()
+        self._macro_instance = Macro(flags={"Get": False, "Return": False})
         self.__magicclass_parent__: None | BaseGui = None
         self.__magicclass_children__: list[MagicTemplate] = []
         self._close_on_run = close_on_run
         self._popup_mode = popup_mode
         self._error_mode = error_mode
+        self._my_symbol = UI
 
-def _get_widget_name(widget):
+def _get_widget_name(widget: Widget):
     # To escape reference
     return widget.name
     
@@ -540,7 +548,6 @@ def _temporal_function_gui_callback(bgui: MagicTemplate, fgui: FunctionGuiPlus,
         return_type = fgui.return_annotation
         result_required = return_type is not inspect._empty
         result = Symbol("result")
-        
         # Standard button will be connected with two callbacks.
         # 1. Build FunctionGui
         # 2. Emit value changed signal.
@@ -657,7 +664,6 @@ def _raise_error_in_msgbox(_func: Callable, parent: Widget = None):
     """
     If exception happened inside function, then open a message box.
     """    
-    @functools_wraps(_func)
     def wrapped_func(*args, **kwargs):
         from qtpy.QtWidgets import QMessageBox
         try:
@@ -673,7 +679,6 @@ def _identity_wrapper(_func: Callable, parent: Widget = None):
     """
     Do nothing.
     """    
-    @functools_wraps(_func)
     def wrapped_func(*args, **kwargs):
         return _func(*args, **kwargs)
     return wrapped_func
@@ -738,4 +743,60 @@ def _field_as_getter(self, bound_value: MagicField):
     return _func
 
 def _need_record(func: Callable):
-    return get_additional_option(func, "record", True)
\ No newline at end of file
+    return get_additional_option(func, "record", True)
+
+def value_widget_callback(gui: MagicTemplate, widget: ValueWidget, name: str, getvalue: bool = True):
+    sym_name = Symbol(name)
+    sym_value = Symbol("value")
+    def _set_value():
+        if not widget.enabled:
+            # If widget is read only, it means that value is set in script (not manually).
+            # Thus this event should not be recorded as a macro.
+            return None
+        
+        gui.changed.emit(gui)
+        
+        if getvalue:
+            sub = Expr(head=Head.getattr, args=[sym_name, sym_value]) # name.value
+        else:
+            sub = Expr(head=Head.value, args=[sym_name])
+        
+        # Make an expression of
+        # >>> x.name.value = value
+        # or
+        # >>> x.name = value
+        expr = Expr(head=Head.assign, 
+                    args=[Expr(head=Head.getattr, 
+                               args=[symbol(gui), sub]), 
+                          widget.value])
+        
+        last_expr = gui._recorded_macro[-1]
+        if (len(gui._recorded_macro) > 1 and 
+            last_expr.head == expr.head and 
+            last_expr.args[0].args[1].head == expr.args[0].args[1].head and
+            last_expr.args[0].args[1].args[0] == expr.args[0].args[1].args[0]):
+            gui._recorded_macro[-1] = expr
+        else:
+            gui._recorded_macro.append(expr)
+        return None
+    return _set_value
+
+def nested_function_gui_callback(gui: MagicTemplate, fgui: FunctionGui):
+    fgui_name = Symbol(fgui.name)
+    def _after_run():
+        inputs = get_parameters(fgui)
+        args = [Expr(head=Head.kw, args=[Symbol(k), v]) for k, v in inputs.items()]
+        # args[0] is self
+        sub = Expr(head=Head.getattr, args=[symbol(gui), fgui_name]) # {x}.func
+        expr = Expr(head=Head.call, args=[sub] + args[1:]) # {x}.func(args...)
+
+        if fgui._auto_call:
+            # Auto-call will cause many redundant macros. To avoid this, only the last input
+            # will be recorded in magic-class.
+            last_expr = gui._recorded_macro[-1]
+            if last_expr.head == Head.call and last_expr.args[0].head == Head.getattr and \
+                last_expr.args[0].args[1] == expr.args[0].args[1]:
+                gui._recorded_macro.pop()
+
+        gui._recorded_macro.append(expr)
+    return _after_run
\ No newline at end of file
