@@ -1,18 +1,17 @@
 from __future__ import annotations
 from inspect import signature
-from typing import Any
+from typing import Any, Callable
 from qtpy.QtWidgets import QMenuBar, QWidget
 from qtpy.QtCore import Qt
 from magicgui.widgets import Container, MainWindow,Label, FunctionGui, Image, Table
 from magicgui.widgets._bases import Widget, ButtonWidget, ValueWidget, ContainerWidget
 from magicgui.widgets._concrete import _LabeledWidget
-
-from ..signature import get_additional_option
+from macrokit import Symbol
 
 from .mgui_ext import PushButtonPlus
 from .menu_gui import MenuGui, ContextMenuGui
-from ._base import BaseGui, PopUpMode, ErrorMode
-from .utils import get_parameters, define_callback, MagicClassConstructionError
+from ._base import BaseGui, PopUpMode, ErrorMode, value_widget_callback, nested_function_gui_callback
+from .utils import define_callback, MagicClassConstructionError
 from ._containers import (
     ButtonContainer,
     GroupBoxContainer,
@@ -26,10 +25,10 @@ from ._containers import (
     ToolBoxContainer
     )
 
-from ..macro import Expr, Head, Symbol, symbol
 from ..utils import iter_members, extract_tooltip
 from ..widgets import FreeWidget
 from ..fields import MagicField
+from ..signature import get_additional_option
 
 class ClassGuiBase(BaseGui):
     # This class is always inherited by @magicclass decorator.
@@ -62,7 +61,7 @@ class ClassGuiBase(BaseGui):
                 
             if hasattr(widget, "value") and fld.record:
                 # By default, set value function will be connected to the widget.
-                f = _value_widget_callback(self, widget, name, getvalue=type(fld) is MagicField)
+                f = value_widget_callback(self, widget, name, getvalue=type(fld) is MagicField)
                 widget.changed.connect(f)
                 
         return widget
@@ -114,6 +113,9 @@ class ClassGuiBase(BaseGui):
                     # convert class method into instance method
                     widget = getattr(self, name, None)
                 
+                if isinstance(widget, BaseGui):
+                    widget._my_symbol = Symbol(name)
+                            
                 if isinstance(widget, MenuGui):
                     # Add menubar to container
                     widget.__magicclass_parent__ = self
@@ -134,10 +136,16 @@ class ClassGuiBase(BaseGui):
                         )
                     _hist.append((name, type(attr), "ContextMenuGui"))
                 
-                elif isinstance(widget, Widget) or callable(widget):
-                    if (not isinstance(widget, Widget)) and callable(widget):
-                        # Methods (FunctionGui not included)
-                        widget = self._create_widget_from_method(widget)
+                elif isinstance(widget, (Widget, Callable)):
+                    if (not isinstance(widget, Widget)) and isinstance(widget, Callable):
+                        # Methods or any callable objects, but FunctionGui is not included.
+                        # NOTE: Here any custom callable objects could be given. Some callable
+                        # objects can be incompatible (like "Signal" object in magicgui) but
+                        # useful. Those callable objects should be passed from widget construction.
+                        try:
+                            widget = self._create_widget_from_method(widget)
+                        except Exception:
+                            continue
                     
                     elif hasattr(widget, "__magicclass_parent__") or \
                         hasattr(widget.__class__, "__magicclass_parent__"):
@@ -148,7 +156,7 @@ class ClassGuiBase(BaseGui):
 
                     elif isinstance(widget, FunctionGui):
                         # magic-class has to know when the nested FunctionGui is called.
-                        f = _nested_function_gui_callback(self, widget)
+                        f = nested_function_gui_callback(self, widget)
                         widget.called.connect(f)
                         
                     else:
@@ -239,8 +247,12 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
                 
             if isinstance(widget, ValueWidget):
                 widget.changed.connect(lambda: self.changed.emit(self))
-            if isinstance(widget, ClassGuiBase) and self._remove_child_margins:
-                widget.margins = (0, 0, 0, 0)
+            
+            if hasattr(widget, "__magicclass_parent__") or \
+                hasattr(widget.__class__, "__magicclass_parent__"):
+                if isinstance(widget, ClassGuiBase) and self._remove_child_margins:
+                    widget.margins = (0, 0, 0, 0)
+                widget.__magicclass_parent__ = self
                 
             _widget = widget
 
@@ -342,60 +354,7 @@ class GroupBoxClassGui: pass
 @make_gui(MainWindow)
 class MainWindowClassGui: pass
 
-def _nested_function_gui_callback(cgui: ClassGuiBase, fgui: FunctionGui):
-    def _after_run():
-        inputs = get_parameters(fgui)
-        args = [Expr(head=Head.kw, args=[Symbol(k), v]) for k, v in inputs.items()]
-        # args[0] is self
-        sub = Expr(head=Head.getattr, 
-                   args=[cgui._recorded_macro[0].args[0], Symbol(fgui.name)]) # {x}.func
-        expr = Expr(head=Head.call, args=[sub] + args[1:]) # {x}.func(args...)
-
-        if fgui._auto_call:
-            # Auto-call will cause many redundant macros. To avoid this, only the last input
-            # will be recorded in magic-class.
-            last_expr = cgui._recorded_macro[-1]
-            if last_expr.head == Head.call and last_expr.args[0].head == Head.getattr and \
-                last_expr.args[0].args[1] == expr.args[0].args[1]:
-                cgui._recorded_macro.pop()
-
-        cgui._recorded_macro.append(expr)
-    return _after_run
-
-def _value_widget_callback(cgui: ClassGuiBase, widget: ValueWidget, name: str, getvalue: bool = True):
-    def _set_value():
-        if not widget.enabled:
-            # If widget is read only, it means that value is set in script (not manually).
-            # Thus this event should not be recorded as a macro.
-            return None
-        
-        cgui.changed.emit(cgui)
-        
-        if getvalue:
-            sub = Expr(head=Head.getattr, args=[Symbol(name), Symbol("value")]) # name.value
-        else:
-            sub = Expr(head=Head.value, args=[Symbol(name)])
-        
-        # Make an expression of
-        # >>> x.name.value = value
-        # or
-        # >>> x.name = value
-        expr = Expr(head=Head.assign, 
-                    args=[Expr(head=Head.getattr, 
-                               args=[symbol(cgui), sub]), 
-                          widget.value])
-        
-        last_expr = cgui._recorded_macro[-1]
-        if (last_expr.head == expr.head and 
-            last_expr.args[0].args[1].head == expr.args[0].args[1].head and
-            last_expr.args[0].args[1].args[0] == expr.args[0].args[1].args[0]):
-            cgui._recorded_macro[-1] = expr
-        else:
-            cgui._recorded_macro.append(expr)
-        return None
-    return _set_value
-
 def _define_context_menu(contextmenu, parent):
     def rightClickContextMenu(point):
         contextmenu.native.exec_(parent.mapToGlobal(point))
-    return rightClickContextMenu
\ No newline at end of file
+    return rightClickContextMenu
