@@ -5,8 +5,8 @@ These widgets are all compatible with the ``append`` method of Container widgets
 
 from magicgui.widgets import * # to avoid importing both magicgui.widgets and magicclass.widgets
 
-from .listwidget import ListWidget
-from .misc import Figure, ConsoleTextEdit, MacroEdit, CheckButton, show_messagebox
+from .pywidgets import ListWidget, DictWidget
+from .misc import Figure, ConsoleTextEdit, MacroEdit, CheckButton
 from .separator import Separator
 from .sequence import ListEdit, TupleEdit
 from .threading import ProgressWidget, progress
@@ -43,16 +43,25 @@ except ImportError:
           "   $ conda install pyqtgraph -c conda forge"
     QtPlotCanvas = NotInstalled(msg.format("QtPlotCanvas"))
     QtImageCanvas = NotInstalled(msg.format("QtImageCanvas"))
-    
+
+try:
+    from .napari import NapariCanvas
+except ImportError:
+    msg = "Module 'napari' is not installed. To use NapariCanvas, " \
+          "you have to install it by:\n" \
+          "   $ pip install napari[all]\n" \
+          "or\n" \
+          "   $ conda install napari -c conda-forge"
+    NapariCanvas = NotInstalled(msg)
 
 __all__ = ["ListWidget", 
            "Figure",
            "ConsoleTextEdit", 
            "MacroEdit",
            "CheckButton",
-           "show_messagebox",
            "Separator",
            "ListEdit",
+           "DictWidget",
            "TupleEdit",
            "ProgressWidget",
            "progress",
@@ -60,4 +69,5 @@ __all__ = ["ListWidget",
            "QtConsole",
            "QtPlotCanvas",
            "QtImageCanvas",
+           "NapariCanvas",
            ]
